//////////////////////////////////////////////////////////////////////////////
// File: test.c
//
// $URL: https://svn.home.lan/winlib/trunk/code/libtestfw/test.c $
// $Id: test.c 33 2010-05-20 20:11:37Z jcurl $
//
// Description
//
// Revision History:
//
//  The revision history contains the date of a change, the author of the
//  change and in the description at a minimum the Problem Report that
//  required the change.
//
//  Date    Author   Description
//  ------- -------- ---------------------------------------------------------
//  15Nov07 a15595   Initial Version, based on mofo/libmofo/test/test.c,v 1.4
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////

#include <config.h>

// Standard system includes
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

// Library includes
#include <winlib/types.h>

// Testing includes
#include <test.h>

//////////////////////////////////////////////////////////////////////////////
// Global Defines (this module only)
//////////////////////////////////////////////////////////////////////////////
#define BAR     "----------------------------------------" \
                "---------------------------------------"

#define CHECK   "CHECK"
#define FAIL    "FAIL"
#define PASS    "PASS"
#define BLOCKED "BLOCKED"

//////////////////////////////////////////////////////////////////////////////
// Type Definitions
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// External Objects Definition
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Static Objects
//////////////////////////////////////////////////////////////////////////////
static int    testnum = 1;
static int    subtestnum;
static int    pass = 0;
static int    fail = 0;
static int    blocked = 0;
static int    unknown = 0;
static int    linepos = 0;
static int    lastresult = TEST_UNKNOWN;
static bool_t testinprogress = FALSE;

//////////////////////////////////////////////////////////////////////////////
// Function Declarations
//////////////////////////////////////////////////////////////////////////////
static void testresult(const char *s);

//////////////////////////////////////////////////////////////////////////////
// Function Definitions
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void teststart(void)
{
    testnum = 1;
    pass = 0;
    fail = 0;
    blocked = 0;
    unknown = 0;
    testinprogress = FALSE;
    lastresult = TEST_UNKNOWN;
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void testfinish(void)
{
    int cases;

    cases = pass+fail+unknown+blocked;
    testinprogress = FALSE;

    printf("%s\n", BAR);
    printf("\nSummary: %4d Test cases run\n\n", cases);
    if (pass+fail > 0) {
        printf("             %4d Passed  (%3d%%)\n", pass, 
               (int)(100.0*pass/(cases)+0.5));
        printf("             %4d Unknown (%3d%%)\n", unknown,
               (int)(100.0*unknown/(cases)+0.5));
        printf("             %4d Blocked (%3d%%)\n", blocked,
               (int)(100.0*blocked/(cases)+0.5));
        printf("             %4d Failed  (%3d%%)\n", fail,
               (int)(100.0*fail/(cases)+0.5));
    }
    printf("\n%s\n", BAR);
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void testcase(const char *testname, ...)
{
    va_list arg;
    va_start(arg, testname);

    printf("%s\n", BAR);
    linepos = printf("%3d. ", testnum++);
    linepos += vprintf(testname, arg);
    subtestnum = 0;

    fflush(stdout);
    testinprogress = TRUE;
    lastresult = TEST_UNKNOWN;
    va_end(arg);
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void subtestcase(const char *testname, ...)
{
    va_list arg;
    va_start(arg, testname);

    if (subtestnum == 0) {
        printf("\n");
    }
    if (subtestnum < 26) {
        linepos = printf("      %c. ", subtestnum + 'a');
    } else {
        linepos = printf("     %c%c. ", 
                         subtestnum / 26 - 1 + 'a', 
                         subtestnum % 26 + 'a');
    }
    subtestnum++;
    linepos += vprintf(testname, arg);
    fflush(stdout);
    testinprogress = TRUE;
    lastresult = TEST_UNKNOWN;
    va_end(arg);
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void testpass(void)
{
    if (!testinprogress) {
        return;
    }

    testresult(PASS);
    pass++;
    testinprogress = FALSE;
    lastresult = TEST_PASSED;
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void testcheck(void)
{
    if (!testinprogress) {
        return;
    }

    testresult(CHECK);
    fflush(stdout);
    unknown++;
    testinprogress = FALSE;
    lastresult = TEST_CHECK;
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void testfail(const char *reason, ...)
{
    va_list arg;

    va_start(arg, reason);
    
    if (testinprogress) {
        testresult(FAIL);
        fail++;
    }

    if (subtestnum) {
        printf("  ");
    }
    printf("     -- ");
    vprintf(reason, arg);
    printf("\n");
    testinprogress = FALSE;
    lastresult = TEST_FAILED;
    va_end(arg);
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
void testblocked(const char *reason, ...)
{
    va_list arg;

    va_start(arg, reason);
    
    if (testinprogress) {
        testresult(BLOCKED);
        blocked++;
    }

    if (reason && strlen(reason)) {
        if (subtestnum) {
            printf("  ");
        }
        printf("     -- ");
        vprintf(reason, arg);
        printf("\n");
    }
    testinprogress = FALSE;
    lastresult = TEST_BLOCKED;
    va_end(arg);
}

//////////////////////////////////////////////////////////////////////////////
// Function: 
//
// Inputs:
//
// Outputs:
//
// Description:
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
static void testresult(const char *s)
{
    int i;

    if (linepos > strlen(BAR) - strlen(s)) {
        printf("\n");
        linepos = 0;
    }

    for (i=linepos; i<strlen(BAR) - strlen(s); i++) {
        printf(" ");
    }
    
    printf("%s\n", s);
    fflush(stdout);
}

//////////////////////////////////////////////////////////////////////////////
// Function: testfailed
//
// Inputs:
//
// Outputs:
//
// Description:
//  Checks if the current test case has passed or failed
//
// Design:
//
// Results:
//
// Error Handling:
//
// To Do:
//
//////////////////////////////////////////////////////////////////////////////
tstate_t teststate()
{
    return lastresult;

}
