# $URL: https://svn.home.lan/winlib/trunk/code/m4/package.m4 $
# $Id: package.m4 51 2013-04-27 18:38:25Z jcurl $

# LX_PACKAGE_EMAIL
# ----------------
# Checks system information for the name of the local account (and assumes
# this is the email address).
#
# Outputs:
#  email - set to the email address
#  PACKAGE_BUILD_EMAIL in the header file "Built by $email"
AC_DEFUN([LX_PACKAGE_EMAIL],
  [AC_REQUIRE([LX_PROG_ID_CURRENT_USER])
   AC_MSG_CHECKING([for local account name])
   nodename=`uname -n`
   if test x$ac_cv_prog_id = xyes; then
      username=`$ID $IDFLAGS`
   else
      username='<unknown>'
   fi
   email=$username\@$nodename
   AC_DEFINE_UNQUOTED(PACKAGE_BUILD_EMAIL, "Built by $email",
      [Email of user building software])
   AC_MSG_RESULT($email)
  ])

# LX_PACKAGE_MACHINE
# ------------------
# Really simple, just makes sure that AC_CANONICAL_BUILD has run and sets an
# appropriate variable in the header (PACKAGE_BUILD_MACHINE)
AC_DEFUN([LX_PACKAGE_MACHINE],
  [if test x$build_os = x; then
      AC_MSG_WARN([
**
** Please contact the developer, they haven't set up they're 'configure' 
** script correctly and tell them about this warning. Thankyou!])
      AC_MSG_CHECKING([for system description])
      AC_DEFINE(PACKAGE_BUILD_MACHINE, [unknown])
   else
      AC_MSG_CHECKING([for system description])
      system="$build_cpu-$build_vendor-$build_os"
      AC_DEFINE_UNQUOTED(PACKAGE_BUILD_MACHINE, "$system",
         [Machine software was built on])
   fi
   AC_MSG_RESULT($system)
  ])

# LX_PACKAGE_TIME
# ---------------
# Get's the current time and puts that into the header file
AC_DEFUN([LX_PACKAGE_TIME],
  [AC_MSG_CHECKING([for current time])
   date=`date`
   AC_DEFINE_UNQUOTED(PACKAGE_BUILD_DATE, "$date", [Date configure was run])
   AC_MSG_RESULT($date)
  ])

# LX_PACKAGE_VERSION
# ------------------
# Redefines stuff from AC_INIT for package info version based on a script
AC_DEFUN([LX_PACKAGE_VERSION],
  [if test x$PERL = x; then
     AC_PATH_PROG(PERL,perl,perl)
   fi
   if test x$PERL = x; then
     AC_MSG_ERROR([You need to have a working version of perl])
   fi
   AC_MSG_CHECKING([for version to use])
   AC_SUBST([CONFIG_STATUS_DEPENDENCIES], 
     ['$(top_srcdir)/version.pl $(top_srcdir)/version.ini'])
   VERSION_EXE="$PERL ${srcdir}/version.pl"
   VERSION_DIR="${srcdir}"
   if test x$RELEASE = xyes || test x$RELEASE = xYES; then
     VERSION=`$VERSION_EXE --release "$VERSION_DIR"`
   else
     VERSION=`$VERSION_EXE "$VERSION_DIR"`
   fi
   PACKAGE_VERSION=$VERSION
   PACKAGE_RCSNOTE=`$VERSION_EXE --note "$VERSION_DIR"`
   PACKAGE_STRING="$PACKAGE_NAME $PACKAGE_VERSION"
   AC_DEFINE_UNQUOTED([PACKAGE_VERSION], "$PACKAGE_VERSION", [Package Version])
   AC_DEFINE_UNQUOTED([PACKAGE_STRING], "$PACKAGE_NAME $VERSION", 
     [Package string])
   AC_DEFINE_UNQUOTED([VERSION], "$VERSION", [Version number of package])
   AC_SUBST([PACKAGE_VERSION])
   AC_SUBST([PACKAGE_STRING])
   AC_SUBST([VERSION])
   AC_MSG_RESULT([$PACKAGE_VERSION ($PACKAGE_RCSNOTE)])
  ])

# LX_PACKAGE_COPYRIGHT
# --------------------
# Defines the copyright that can be exported along with company and
# trademark.
AC_DEFUN([LX_PACKAGE_COPYRIGHT],
  [PACKAGE_COPYRIGHT="$1"
   PACKAGE_COMPANY="$2"
   PACKAGE_TRADEMARK="$3"
   AC_SUBST([PACKAGE_COPYRIGHT])
   AC_SUBST([PACKAGE_COMPANY])
   AC_SUBST([PACKAGE_TRADEMARK])
   AC_COPYRIGHT([$PACKAGE_COPYRIGHT])
  ])

# LX_PACKAGE_OBJVERSION([LIBNAME], [CURRENT], [REVISION], [AGE], [LA FILE])
# -------------------------------------------------------------------------
# For a particular library, define 5 variables:
#  LIBNAME_VERSION_INFO
#  LIBNAME_MAJOR
#  LIBNAME_MINOR
#  LIBNAME_REVISION
#  LIBNAME_BUILD
#  LIBNAME_FILENAME
#
# We will probably break on the following:
#   irix, nonstopux, osf (versioning schemes)
#   beos, pw32*
#
# We don't support -release or other versioning schemes. We assume
# -version-info c:r:a
#
# Tested on:
#  - sparc-sun-solaris2.10
#  - i686-pc-linux-gnu
#  - i686-pc-cygwin
#  - i686-pc-mingw32
AC_DEFUN([LX_PACKAGE_OBJVERSION],
  [if test x$PERL = x; then
     AC_PATH_PROG(PERL,perl,perl)
   fi
   if test x$PERL = x; then
     AC_MSG_ERROR([You need to have a working version of perl])
   fi
   AC_SUBST([CONFIG_STATUS_DEPENDENCIES], 
     ['$(top_srcdir)/version.pl $(top_srcdir)/version.ini'])
   VERSION_EXE="$PERL ${srcdir}/version.pl"
   VERSION_DIR="${srcdir}"
   $1_VERSION_INFO="$2:$3:$4"

   outputname=$5
   case $outputname in
   lib*)
     name=`echo $outputname | sed -e 's/\.la$//' -e 's/^lib//'`
     eval shared_ext=\"$shrext_cmds\"
     eval libname=\"$libname_spec\"
     ;;
   *)
     name=$outputname
	 libname=""
     ;;
   esac

   release=""
   current=$2
   revision=$3
   age=$4

   if test "x$libname" != "x"; then
     AC_MSG_CHECKING([library version $1])
     case $version_type in
     darwin)
       major=.`expr $current - $age`
       versuffix="$major.$age.$revision"
       ;;
     freebsd-aout)
       major=".$current"
       versuffix=".$current.$revision";
       ;;
     freebsd-elf)
       major=".$current"
       versuffix=".$current";
       ;;
     linux)
       major=.`expr $current - $age`
       versuffix="$major.$age.$revision"
       ;;
     sunos)
       major=".$current"
       versuffix=".$current.$revision"
       ;;
     windows)
       major=`expr $current - $age`
       versuffix="-$major"
       ;;
     qnx)
       major=".$current"
       versuffix=".$current";
       ;;
     *)
       AC_MSG_WARN([Your library versioning scheme '$version_type' isn't supported])
       major=""
       versuffix=""
       ;;
     esac

     # Generate the filename $1_FILENAME
     case $target_os in
     *cygwin* | mingw*)
       eval $1_FILENAME=\"$soname_spec\"
       ;;
     *)
       eval library_names=\"$library_names_spec\"
       set dummy $library_names
       $1_FILENAME="${2}"
       ;;
     esac

     $1_MAJOR=`expr $current - $age`
     $1_MINOR=$age
     $1_REVISION=$revision

     PACKAGE_BUILD=`$VERSION_EXE --build "$VERSION_DIR"`
     $1_BUILD="$PACKAGE_BUILD"
     version=`echo ${$1_MAJOR}.${$1_MINOR}.${$1_REVISION}.${$1_BUILD}`
   fi

   if test "x$libname" = "x"; then
     AC_MSG_CHECKING([file version $1])
     # Generate the filename $1_FILENAME
     case $target_os in
     *cygwin* | mingw*)
       eval $1_FILENAME="${name}.exe"
       ;;
     *)
       $1_FILENAME="$name"
       ;;
     esac

     $1_MAJOR=$current
	 $1_MINOR=$age
	 $1_REVISION=$revision
     PACKAGE_BUILD=`$VERSION_EXE --build "$VERSION_DIR"`
     $1_BUILD="$PACKAGE_BUILD"
     version=`echo ${$1_MAJOR}.${$1_MINOR}.${$1_REVISION}.${$1_BUILD}`
   fi
   
   AC_SUBST([$1_VERSION_INFO])
   AC_SUBST([$1_MAJOR])
   AC_SUBST([$1_MINOR])
   AC_SUBST([$1_REVISION])
   AC_SUBST([$1_BUILD])
   AC_SUBST([$1_FILENAME])
   AC_MSG_RESULT([${$1_FILENAME} ($version)])
  ])
